﻿Imports Microsoft.Win32
Imports System.Runtime.InteropServices

Public Class MainWindow

    Declare Auto Sub AddClipboardFormatListener Lib "user32.dll" _
     Alias "AddClipboardFormatListener" (ByVal hwnd As IntPtr)

    Declare Auto Sub RemoveClipboardFormatListener Lib "user32.dll" _
     Alias "RemoveClipboardFormatListener" (ByVal hwnd As IntPtr)

    Public strSoftwareTitle As String
    Public strVersion As String
    Public strCopyright As String
    Public Const strBetaVer As String = ""    ' ←β版の場合は"β", α版の場合は"α", 何でも無い場合は "" にしておくこと
    Public Const strURL As String = "http://www.inasoft.org/"
    Public Const strRegName As String = "Software\InaSoftAns\CaptureSaveAs\CurrentVersion"
    Public Const strHelpFileName As String = ".\CaptureSaveAs.txt"

    ' 一時的にオフになっているか
    Public Shared bPause As Boolean = False

    ' [設定]
    ' 保存形式(Jpeg or PNG)
    Public Shared intFormat As Integer = 0
    ' 簡易指定になっているか？　詳細指定になっているか？
    Public Shared bolShosai As Boolean = False
    ' クリップボード読み出しディレイ
    Public Shared intDelay As Integer = 0

    ' [設定] - 簡易指定
    ' 保存先のフォルダ名＋ファイル名のベース
    Public Shared strFileName As String = ""
    ' ファイル名に日付時刻を付与するか
    Public Shared bAddDateTime As Boolean = True
    ' ファイル名に連番を付けるか
    Public Shared bAddNumber As Boolean = True
    Public Shared intNumber As Integer = 1

    ' [設定] - 詳細指定
    ' 保存先のフォルダ名＋ファイル名のフォーマット
    Public Shared strFileName2 As String = ""

    ' [一般]
    ' 保存ダイアログは１つしか開かないようにする
    Public Shared bolSingle As Boolean = False

    ' 初回起動？
    Public Shared bFirstBoot As Boolean = False
    ' クリップボード監視中？
    Public Shared bClipboardListening As Boolean = False

    ' キャプチャウィンドウのプール
    Public Shared CaptureWindowPool As New Dictionary(Of ULong, CaptureWindow)

    ' バージョン情報取得
    Private Sub GetVersionInfo()
        'AssemblyTitleの取得
        Dim asmttl As System.Reflection.AssemblyTitleAttribute = _
            CType(Attribute.GetCustomAttribute( _
                System.Reflection.Assembly.GetExecutingAssembly(), _
                GetType(System.Reflection.AssemblyTitleAttribute)),  _
                    System.Reflection.AssemblyTitleAttribute)
        strSoftwareTitle = asmttl.Title

        'AssemblyCopyrightの取得
        Dim asmcpy As System.Reflection.AssemblyCopyrightAttribute = _
            CType(Attribute.GetCustomAttribute( _
                System.Reflection.Assembly.GetExecutingAssembly(), _
                GetType(System.Reflection.AssemblyCopyrightAttribute)),  _
                    System.Reflection.AssemblyCopyrightAttribute)
        strCopyright = asmcpy.Copyright + vbCrLf + vbCrLf + "http://www.inasoft.org/"

        '自分自身のバージョン情報を取得する
        Dim ver As System.Diagnostics.FileVersionInfo = _
            System.Diagnostics.FileVersionInfo.GetVersionInfo( _
            System.Reflection.Assembly.GetExecutingAssembly().Location)
        strVersion = "" & ver.FileMajorPart & "." & ver.FileMinorPart & ver.FileBuildPart & strBetaVer

    End Sub

    Private Sub MainWindow_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        '二重起動をチェックする
        If Diagnostics.Process.GetProcessesByName( _
            Diagnostics.Process.GetCurrentProcess.ProcessName).Length > 1 Then
            'すでに起動していると判断する
            Me.NotifyIcon1.Dispose()
            MessageBox.Show("多重起動はできません。", strSoftwareTitle, MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Close()
            Return
        End If

        ' バージョン情報の取得
        GetVersionInfo()

        ' はじめて本ソフトを開いたときは、保存先を設定させるためにデフォルトで表示
        Try
            ' レジストリキーのパスを指定してレジストリを開く
            Dim rKey As RegistryKey = Registry.CurrentUser.OpenSubKey(strRegName)
            If Not rKey Is Nothing Then
                ' レジストリの値を取得
                With rKey
                    ' 簡易指定か詳細指定か
                    If (CInt(.GetValue("Shosai", 0)) = 0) Then
                        bolShosai = False
                    Else
                        bolShosai = True
                    End If

                    ' ファイル名のベース[簡易指定の場合]
                    strFileName = CStr(.GetValue("FileNameBase", ""))

                    ' ファイル名のフォーマット[詳細指定の場合]
                    strFileName2 = CStr(.GetValue("FileNameFormat", ""))

                    ' 日付時刻付与[簡易指定の場合]
                    If (CInt(.GetValue("AddDateTime", 1)) = 0) Then
                        bAddDateTime = False
                    Else
                        bAddDateTime = True
                    End If

                    ' 連番を付けるか[簡易指定の場合]
                    If (CInt(.GetValue("AddNumber", 1)) = 0) Then
                        bAddNumber = False
                    Else
                        bAddNumber = True
                    End If

                    ' 保存形式
                    If (CInt(.GetValue("SaveFormat", 0)) = 0) Then
                        intFormat = 0 ' Jpeg
                    Else
                        intFormat = 1 ' Png
                    End If

                    ' 保存ダイアログは１つしか開かないようにする
                    If (CInt(.GetValue("Single", 0)) = 0) Then
                        bolSingle = False
                    Else
                        bolSingle = True
                    End If

                    ' クリップボード読み出しディレイ
                    intDelay = CInt(.GetValue("Delay", 0))

                    .Close()    ' 開いたレジストリキーを閉じる
                End With
            End If
        Catch ex As Exception
            ' 読み込みに失敗
            ' 失敗でも、とりあえず何もしない
            ' (後にファイル名が空だった場合のルートがあるので、そこにゆだねる)
        End Try

        ' 過去β版との互換のため、値が存在しなければ別に初期化
        If strFileName2 = "" Then
            strFileName2 = System.Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory) + "\Base-<%t4y><%t0m><%t0d>-<%t0h><%t0n><%t0s>-<%n8>-*"
        End If

        ' ファイル名が空だった場合は、初回と見なし、設定ダイアログを開く
        ' 初期値はデスクトップ\Base-
        If strFileName = "" Then
            bFirstBoot = True
            strFileName = System.Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory) + "\Base-"
            miSettings_Click(sender, e)
            If bFirstBoot = True Then
                ' キャンセルされた
                Me.Close()
                Return
            End If
        End If

        ' クリップボード監視の開始
        AddClipboardFormatListener(Me.Handle)
        bClipboardListening = True
    End Sub

    Private Sub ContextMenuStrip1_Opening(sender As System.Object, e As System.ComponentModel.CancelEventArgs) Handles ContextMenuStrip1.Opening
        ' メニューが開いたら、一時停止状態をチェック状態として反映
        miPause.Checked = bPause
    End Sub

    Private Sub miSettings_Click(sender As System.Object, e As System.EventArgs) Handles miSettings.Click
        ' 設定ダイアログを開く
        Static f1 As Form1 = Nothing

        If f1 Is Nothing Then
            ' まだ表示していなければロードして表示
            f1 = New Form1
            f1.ShowDialog()
            f1 = Nothing
        Else
            ' すでに設定ダイアログが表示済みなら、最前面に表示するだけ
            f1.TopMost = True
            f1.Refresh()
            f1.TopMost = False
        End If

    End Sub

    Private Sub miPause_Click(sender As System.Object, e As System.EventArgs) Handles miPause.Click
        ' 一時停止状態の変更
        bPause = Not bPause
        If bPause Then
            Me.NotifyIcon1.Icon = My.Resources.CaptureSaveAsPause
        Else
            Me.NotifyIcon1.Icon = Form1.Icon
        End If
    End Sub

    Private Sub miHelp_Click(sender As System.Object, e As System.EventArgs) Handles miHelp.Click
        ' ドキュメントの表示
        Try
            System.Diagnostics.Process.Start(strHelpFileName)
        Catch ex As Exception
            ErrorDialog.ShowMessage(strHelpFileName + "を開けません。", ex.ToString(), "ドキュメントを開く")
        End Try
    End Sub

    Private Sub miAbout_Click(sender As System.Object, e As System.EventArgs) Handles miAbout.Click
        ' バージョン情報の表示
        Static dlgAboutBox As AboutDialog = Nothing

        If dlgAboutBox Is Nothing Then
            ' まだ表示していなければロードして表示
            dlgAboutBox = New AboutDialog
            dlgAboutBox.ShowDialog()
            dlgAboutBox = Nothing
        Else
            ' すでに設定ダイアログが表示済みなら、最前面にするだけ
            dlgAboutBox.TopMost = True
            dlgAboutBox.Refresh()
            dlgAboutBox.TopMost = False
        End If

    End Sub

    Private Sub miExit_Click(sender As System.Object, e As System.EventArgs) Handles miExit.Click
        ' 終了
        If CaptureWindowPool.Count > 0 Then
            If DialogResult.Cancel = MessageBox.Show("未保存の画像があります。終了してもよろしいですか?", strSoftwareTitle, MessageBoxButtons.OKCancel, MessageBoxIcon.Question) Then
                ' キャンセル
                Return
            End If
        End If

        If bClipboardListening Then
            RemoveClipboardFormatListener(Me.Handle)
        End If

        NotifyIcon1.Dispose()
        Me.Close()
    End Sub

    Private Sub NotifyIcon1_MouseDoubleClick(sender As System.Object, e As System.Windows.Forms.MouseEventArgs) Handles NotifyIcon1.MouseDoubleClick
        ' タスクトレイアイコンをダブルクリックしたら、設定を開く
        miSettings_Click(sender, e)
    End Sub

    ' クリップボード監視
    Protected Overrides Sub WndProc(ByRef m As Message)
        If m.Msg = &H31D Then ' クリップボードが変更されたら
            If bPause = False AndAlso Clipboard.ContainsImage() Then
                ' かつ、一時停止中でなく、中身がイメージだったら

                ' ディレイ時間確定（クリップボードを扱う他のプログラムとの相性問題があるときのため）
                Dim DelayMsec As Integer = 0

                Select Case MainWindow.intDelay
                    Case 1
                        DelayMsec = 100

                    Case 2
                        DelayMsec = 200

                    Case 3
                        DelayMsec = 500

                    Case 4
                        DelayMsec = 700

                    Case 5
                        DelayMsec = 1000

                    Case Else
                        DelayMsec = 0
                End Select

                If DelayMsec = 0 Then
                    TimerForDelay_Tick(Nothing, Nothing)
                Else
                    TimerForDelay.Interval = DelayMsec
                    TimerForDelay.Start()
                End If

                ' 処理は即座に返す
                m.Result = IntPtr.Zero
            End If
        Else
            MyBase.WndProc(m)
        End If
    End Sub

    ' ディレイ用タイマ（すぐの場合も呼び出される）
    Private Sub TimerForDelay_Tick(sender As System.Object, e As System.EventArgs) Handles TimerForDelay.Tick
        TimerForDelay.Stop()

        ' キャプチャウィンドウを生成して読み出しをさせる
        ' キャプチャウィンドウはリストにプールしておき、並列動作できるようにしておく
        Static newPrivateNumber As ULong = 0 ' なるべくたっぷり振れるような固有番号

        If bolSingle = True AndAlso CaptureWindowPool.Count > 0 Then
            ' １つの保存ダイアログを開くことしか認められておらず、１つ以上のウィンドウが開いている場合
            ' (とはいえ、ここまでの経緯で複数のウィンドウが既に開かれてしまっているかもしれない。
            '  最終的に開いているウィンドウ位置と入力中の名前を使うことにする)

            ' 最後のキーを取得する
            Dim ulLastKey As ULong
            For Each ulKey As ULong In CaptureWindowPool.Keys
                ulLastKey = ulKey
            Next

            ' 画像の再読み出し
            Dim use_cw As New CaptureWindow
            use_cw = CaptureWindowPool.Item(ulLastKey)
            use_cw.loadImage(Now())

            use_cw.Show()
            use_cw.TopMost = True
            use_cw.Refresh()
            use_cw.TopMost = False

        Else
            Dim new_cw As New CaptureWindow
            new_cw.ulPrivateNumber = newPrivateNumber
            new_cw.Show(Nothing)
            new_cw.TopMost = True
            new_cw.Refresh()
            new_cw.TopMost = False

            ' ウィンドウをリストに追加し、固有番号を記憶する。固有番号はカウントアップ
            CaptureWindowPool.Add(newPrivateNumber, new_cw)
            newPrivateNumber += 1UL

        End If


    End Sub

    ' キャプチャウィンドウのリリース
    Public Shared Sub DelCaptureWindow(ByRef ulPrivateNumber As ULong)
        ' 上で振った固有番号を使ってリリースする
        CaptureWindowPool.Remove(ulPrivateNumber)
    End Sub

    ' アイコンのマウスクリック
    ' (右クリックの場合も、まずはここに反応する)
    ' すでに表示済みのウィンドウを再表示
    Private Sub NotifyIcon1_MouseClick(sender As System.Object, e As System.Windows.Forms.MouseEventArgs) Handles NotifyIcon1.MouseClick
        For Each Values As CaptureWindow In CaptureWindowPool.Values
            Values.TopMost = True
            Values.Refresh()
            Values.TopMost = False
        Next
    End Sub

End Class